﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Data;

namespace TestProject
{
    /// <summary>
    /// 데이터 그리드 컬렉션 뷰
    /// </summary>
    public class DataGridCollectionView : ListCollectionView
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Event
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 항목 수 요청시 이벤트 - ItemCountRequest

        /// <summary>
        /// 항목 수 요청시 이벤트
        /// </summary>
        public event Action<DataGridCollectionView, ItemCountEventArgs> ItemCountRequest = delegate { };

        #endregion
        #region 항목 리스트 요청시 이벤트 - ItemListRequest

        /// <summary>
        /// 항목 리스트 요청시 이벤트
        /// </summary>
        public event Action<DataGridCollectionView, ItemListEventArgs> ItemListRequest = delegate { };

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 페이지 크기
        /// </summary>
        private const int PageSize = 20;

        /// <summary>
        /// 데이터 테이블
        /// </summary>
        private DataTable dataTable;

        /// <summary>
        /// 데이터 행 딕셔너리
        /// </summary>
        Dictionary<int, DataRow> dataRowDictionary = new Dictionary<int, DataRow>();

        /// <summary>
        /// 잠금 여부
        /// </summary>
        private bool isLocked = false;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 카운트 - Count

        /// <summary>
        /// 카운트
        /// </summary>
        public override int Count
        {
            get
            {
                ItemCountEventArgs eventArgs = new ItemCountEventArgs(this);

                ItemCountRequest(this, eventArgs);

                return eventArgs.Count;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - DataGridCollectionView(list)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="list">리스트</param>
        private DataGridCollectionView(IList list) : base(list)
        {
        }

        #endregion
        #region 생성자 - DataGridCollectionView(dataTable)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="dataTable">데이터 테이블</param>
        public DataGridCollectionView(DataTable dataTable) : this(dataTable.DefaultView)
        {
            this.dataTable = dataTable;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 항목 구하기 - GetItemAt(index)

        /// <summary>
        /// 항목 구하기
        /// </summary>
        /// <param name="index">인덱스</param>
        /// <returns>항목</returns>
        public override object GetItemAt(int index)
        {
            if(!this.dataRowDictionary.ContainsKey(index))
            {
                if(this.isLocked)
                {
                    return this.dataTable.NewRow();
                }

                this.isLocked = true;

                Task.Factory.StartNew
                (
                    () =>
                    {
                        try
                        {
                            int startIndex = index;

                            ItemListEventArgs eventArgs = new ItemListEventArgs(this, startIndex, PageSize);

                            ItemListRequest(this, eventArgs);

                            if(eventArgs.ItemList == null)
                            {
                                return;
                            }

                            int i = 0;

                            foreach(DataRow row in eventArgs.ItemList.OfType<DataRow>())
                            {
                                int newIndex = startIndex + i;

                                this.dataRowDictionary[newIndex] = row;

                                i++;
                            }

                            Action action = () => Refresh();

                            Application.Current.Dispatcher.BeginInvoke(action);
                        }
                        catch(Exception)
                        {
                        }

                        this.isLocked = false;
                    }
                );

                return this.dataTable.NewRow();
            }
                       
            return this.dataRowDictionary[index];
        }
        
        #endregion
    }
}