﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;

namespace TestProject
{
    /// <summary>
    /// 메인 윈도우 뷰 모델
    /// </summary>
    public class MainWindowViewModel : INotifyPropertyChanged
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Event
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 속성 변경시 이벤트 - PropertyChanged

        /// <summary>
        /// 속성 변경시 이벤트
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged = delegate { };

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 카운트
        /// </summary>
        private int count = 10000000;

        /// <summary>
        /// 데이터 테이블
        /// </summary>
        private DataTable dataTable;

        /// <summary>
        /// 난수기
        /// </summary>
        private Random random = new Random();

        /// <summary>
        /// 컬렉션 뷰
        /// </summary>
        private DataGridCollectionView collectionView;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 컬렉션 뷰 - CollectionView

        /// <summary>
        /// 컬렉션 뷰
        /// </summary>
        public DataGridCollectionView CollectionView
        {
            get
            {
                return this.collectionView;
            }
            set
            {
                if(this.collectionView != value)
                {
                    this.collectionView = value;

                    FirePropertyChangedEvent(nameof(CollectionView));
                }
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainWindowViewModel()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainWindowViewModel()
        {
            this.dataTable = new DataTable();

            this.dataTable.Columns.Add("Column 1");
            this.dataTable.Columns.Add("Column 2");
            this.dataTable.Columns.Add("Column 3");

            DataGridCollectionView collectionView = new DataGridCollectionView(dataTable);

            collectionView.ItemCountRequest += collectionView_ItemCountRequest;
            collectionView.ItemListRequest  += collectionView_ItemListRequest;

            CollectionView = collectionView;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 컬렉션 뷰 항목 카운트 요청시 처리하기 - collectionView_ItemCountRequest(sender, e)

        /// <summary>
        /// 컬렉션 뷰 항목 카운트 요청시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void collectionView_ItemCountRequest(DataGridCollectionView sender, ItemCountEventArgs e)
        {
            e.Count = this.count;
        }

        #endregion
        #region 컬렉션 뷰 항목 리스트 요청시 처리하기 - collectionView_ItemListRequest(sender, e)

        /// <summary>
        /// 컬렉션 뷰 항목 리스트 요청시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void collectionView_ItemListRequest(DataGridCollectionView sender, ItemListEventArgs e)
        {
            int startIndex       = e.StartIndex;
            int requestItemCount = e.RequestedItemCount;

            List<object> itemList = new List<object>();

            for(int i = startIndex; i < startIndex + requestItemCount; i++)
            {
                DataRow row = dataTable.NewRow();

                row[0] = i.ToString();
                row[1] = random.Next(1000).ToString();
                row[2] = random.Next(1000).ToString();

                itemList.Add(row);
            }

            e.SetItemList(itemList);
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 속성 변경시 이벤트 발생시키기 - FirePropertyChangedEvent(propertyName)

        /// <summary>
        /// 속성 변경시 이벤트 발생시키기
        /// </summary>
        /// <param name="propertyName">속성명</param>
        private void FirePropertyChangedEvent(string propertyName)
        {
            PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }

        #endregion
    }
}